// @ myth - client - macro.h
//
// Myth is a multiuser screen oriented adventure game.
// Copyright (C) 1989 Per Abrahamsen
// 
// This file is part of Myth.
// 
// Myth is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 1, or (at your option) any later
// version.
// 
// Myth is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Myth; see the file COPYING.  If not, write to the Free
// Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
//
// My email address is: amanda@iesd.auc.dk

// @@ begin

#ifndef MACRO_HEADER
#define MACRO_HEADER

// @@ include

#include "common/config.h"
#include "common/command.h"
#include "common/symbol.h"
#include "common/string.h"

// @@ enum key_type

enum key_type
{ KEY_REDRAW, KEY_ESCAPE, KEY_HELP, KEY_LITERAL, KEY_ERASE, KEY_KILL,
  KEY_ACCEPT, KEY_DIRECTION, KEY_CONFIRMATION, KEY_PAGE, KEY_END,
  KEY_ERROR = ERROR };

// @@ class key_link
//
// This type is used by MACRO to keep a stack of waiting keys.

class key_link
{
    char* string;
    char* current;
public:
    key_link* next;
    bool pop();
    inline char top()
    { return *current; }
    key_link(char key, key_link* old);
    key_link(char* macro, key_link* old);
    ~key_link();
};

// @@ class MACRO

class MACRO
{
    struct
    {
	short special[KEY_END];
	from_client command;
	char* macro;
    } 
    map[MaxChar];
    
    key_link* stack;
    bool literal;
    void pop();
public:
    void put(char key);
    inline bool ready()
    { return stack != NULL; }
    bool check(key_type type);
    inline bool argument()
    { return !literal && stack && isdigit(stack->top()); }
    int value();
    symbol_type symbol();
    char key();
    direction dir();
    confirmation conf();
    from_client MACRO::command();
    void edit(char* string);
    MACRO(STRING name);
    ~MACRO();
};

// @@ end

#endif MACRO_HEADER

// @@ emacs
//
// Local Variables:
// mode: c++
// outline-regexp: "// @+ "
// eval: (outline-minor-mode 1)
// End:
