// @ myth - server - armor.h
//
// Myth is a multiuser screen oriented adventure game.
// Copyright (C) 1989 Per Abrahamsen
// 
// This file is part of Myth.
// 
// Myth is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 1, or (at your option) any later
// version.
// 
// Myth is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Myth; see the file COPYING.  If not, write to the Free
// Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
//
// My email address is: amanda@iesd.auc.dk

// @@ begin

#ifndef ARMOR_HEADER
#define ARMOR_HEADER

// @@ include

#include "server/thing.h"

// @@ LEATHER_ARMOR

class LEATHER_ARMOR : public ARMOR
{
    // Identification
    //
protected:
    int protect(int power, damage_type* type);
    int BaseValue()
    { return 100; }
public:
    object_type type()
    { return ARMOR_LEATHER; }
    const char* name()
    { return "leather armor"; }
    long size()
    { return 5L * MyCount; }
    long weight()
    { return 10L * MyCount; }
    int strength()
    { return 9; }
    //
    // Administration
    //
private:
    void create()
    { SetLetter('l'); }
public:
    OBJECT* split(long arg)
    { MyCount -= arg; return new LEATHER_ARMOR(this, arg); }
    LEATHER_ARMOR(BINARY* binary) : (binary)
    { create(); }
    LEATHER_ARMOR(PARSE* parse) : (parse)
    { create(); }
    LEATHER_ARMOR(LEATHER_ARMOR* thing, long count) : (thing, count)
    { create(); }
    LEATHER_ARMOR(long count = 1L) : (count)
    { create(); }
};

// @@ RING_MAIL

class RING_MAIL : public ARMOR
{
    // Identification
    //
protected:
    int protect(int power, damage_type* type);
    int BaseValue()
    { return 250; }
public:
    object_type type()
    { return ARMOR_RING; }
    const char* name()
    { return "ring mail"; }
    long size()
    { return 5L * MyCount; }
    long weight()
    { return 10L * MyCount; }
    int strength()
    { return 9; }
    //
    // Administration
    //
private:
    void create()
    { SetLetter('r'); }
public:
    OBJECT* split(long arg)
    { MyCount -= arg; return new RING_MAIL(this, arg); }
    RING_MAIL(BINARY* binary) : (binary)
    { create(); }
    RING_MAIL(PARSE* parse) : (parse)
    { create(); }
    RING_MAIL(RING_MAIL* thing, long count) : (thing, count)
    { create(); }
    RING_MAIL(long count = 1L) : (count)
    { create(); }
};

// @@ PLATE_MAIL

class PLATE_MAIL : public ARMOR
{
    // Identification
    //
protected:
    int protect(int power, damage_type* type);
    int BaseValue()
    { return 1000; }
public:
    object_type type()
    { return ARMOR_PLATE; }
    const char* name()
    { return "plate mail"; }
    long size()
    { return 5L * MyCount; }
    long weight()
    { return 10L * MyCount; }
    int strength()
    { return 9; }
    //
    // Administration
    //
private:
    void create()
    { SetLetter('p'); }
public:
    OBJECT* split(long arg)
    { MyCount -= arg; return new PLATE_MAIL(this, arg); }
    PLATE_MAIL(BINARY* binary) : (binary)
    { create(); }
    PLATE_MAIL(PARSE* parse) : (parse)
    { create(); }
    PLATE_MAIL(PLATE_MAIL* thing, long count) : (thing, count)
    { create(); }
    PLATE_MAIL(long count = 1L) : (count)
    { create(); }
};

// @@ end

#endif ARMOR_HEADER

// @@ emacs
//
// Local Variables:
// mode: c++
// outline-regexp: "// @+ "
// eval: (outline-minor-mode 1)
// End:
