// @ myth - server - id.h
//
// Myth is a multiuser screen oriented adventure game.
// Copyright (C) 1989 Per Abrahamsen
// 
// This file is part of Myth.
// 
// Myth is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 1, or (at your option) any later
// version.
// 
// Myth is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Myth; see the file COPYING.  If not, write to the Free
// Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
//
// My email address is: amanda@iesd.auc.dk

// @@ begin

#ifndef ID_HEADER
#define ID_HEADER

// @@ include

#include "common/list.h"

struct BINARY;
struct OBJECT;
struct INVENTORY;

// @@ typedef id_type
//
// All objects will have have a unique value of id_type.
//
// Supported commands:
//    operators '==', '!=', '='
//    functions ::compare, ::SaveIdentity, ::LoadIdentity

typedef long id_type;

// @@ const id_type NoIdentity
//
// This value is guaranteed not to be used by any object.
static const id_type NoIdentity = 0L;

// @@ IDENTITY
//
// The object `identity' is used to generate new unique id's

class IDENTITY
{
    id_type id;
public:
    // Return a new id
    id_type operator()()
    { return (id == 0L) ? (id++, id++) : id++; }
    // Load current state
    void load(BINARY* binary);
    // Load new state
    void save(BINARY* binary);
    // When created first time
    IDENTITY()
    { id = 1L; }
};

extern IDENTITY identity;

// @@ id_type LoadIdentity(BINARY* binary)
//
// Read id from file

extern id_type LoadIdentity(BINARY* binary);

// @@ void SaveIdentity(BINARY* binary, id_type id)
//
// Save id to file

extern void SaveIdentity(BINARY* binary, id_type id);

// @@ ID_LIST

class ID_LIST 
{
private:
    // I have now come to the conclusion, that private members is less
    // confusing than private base classes.
    LIST list;
    // Create a loop
    inline void reset()
    { list.reset(); }
    inline void next()
    { list.next(); }
    inline id_type current()
    { return (* (id_type*) list.current()); }
    inline bool done()		// Last element read
    { return list.remain() < 1; }
    // maintain the list
    void insert(id_type id);
    void erase();
public:
    int length()
    { return list.length(); }
    bool member(OBJECT* object);
    void remove(OBJECT* object);
    void add(OBJECT* object);
    void forget(int chance, INVENTORY* inventory);
    void forget(int chance);
    void save(BINARY* binary);
    void load(BINARY* binary);
};

// @@ end

#endif ID_HEADER

// @@ emacs
//
// Local Variables:
// mode: c++
// outline-regexp: "// @+ "
// eval: (outline-minor-mode 1)
// End:
