// @ myth - server - question.h
//
// Myth is a multiuser screen oriented adventure game.
// Copyright (C) 1989 Per Abrahamsen
// 
// This file is part of Myth.
// 
// Myth is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 1, or (at your option) any later
// version.
// 
// Myth is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Myth; see the file COPYING.  If not, write to the Free
// Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
//
// My email address is: amanda@iesd.auc.dk

// @@ begin

#ifndef QUESTION_HEADER
#define QUESTION_HEADER

// @@ include

#include "server/command.h"
#include "server/object.h"

struct SLEEP;
struct RAY;

// @@ enum question_value

enum question_value
{ Q_VALUE_PARSING, Q_VALUE_DONE, Q_VALUE_AGAIN, Q_VALUE_ERROR = ERROR };

// @@ class QUESTION

class QUESTION
{
protected:
    question_state state;	// How much has been read
    long argument;		// Argument from client to command
public:
    bool fresh();
    virtual void parse(CHANNEL* channel);
    virtual question_value play(LEVEL*, PLAYER*);
    virtual QUESTION* next();
    QUESTION();
    virtual ~QUESTION();
};

// @@ class QUESTION_OBJECT

class QUESTION_OBJECT : public QUESTION
{
protected:
    symbol_type symbol;
    char letter;
public:
    void parse(CHANNEL* channel);
    QUESTION_OBJECT();
};

// @@ class QUESTION_LETTER

class QUESTION_LETTER : public QUESTION
{
protected:
    char letter;
public:
    void parse(CHANNEL* channel);
    QUESTION_LETTER();
};

// @@ class QUESTION_POSITION

class QUESTION_POSITION : public QUESTION
{
protected:
    coor position;
public:
    void parse(CHANNEL* channel);
    QUESTION_POSITION();
};

// @@ class QUESTION_COMMAND

class QUESTION_COMMAND : public QUESTION
{
    COMMAND* command;		// The current command
public:
    void parse(CHANNEL* channel);
    question_value play(LEVEL* level, PLAYER* player);
    QUESTION* next();
    QUESTION_COMMAND(CHANNEL* channel);
    ~QUESTION_COMMAND();
};

// @@ class QUESTION_GET

class QUESTION_GET : public QUESTION_OBJECT
{
public:
    question_value play(LEVEL* level, PLAYER* player);
    QUESTION_GET(long arg, CHANNEL* channel);
};

// @@ class QUESTION_EAT

class QUESTION_EAT : public QUESTION_LETTER
{
    QUESTION* TryAgain;
public:
    question_value play(LEVEL* level, PLAYER* player);
    QUESTION* next();
    QUESTION_EAT(CHANNEL* channel);
};

// @@ class QUESTION_IDENTIFY

class QUESTION_IDENTIFY : public QUESTION_OBJECT
{
    int value;
public:
    question_value play(LEVEL*, PLAYER* player);
    QUESTION_IDENTIFY(int val, CHANNEL* channel);
};

// @@ class QUESTION_BLESS

class QUESTION_BLESS : public QUESTION_OBJECT
{
    holy_type holy;
    void CurseWorn(PLAYER* player);
public:
    question_value play(LEVEL*, PLAYER* player);
    QUESTION_BLESS(holy_type h, CHANNEL* channel);
};

// @@ class QUESTION_ENCHANT

class QUESTION_ENCHANT : public QUESTION_OBJECT
{
    int value;
public:
    question_value play(LEVEL*, PLAYER* player);
    QUESTION_ENCHANT(int val, CHANNEL* channel);
};

// @@ class QUESTION_TELEPORT

class QUESTION_TELEPORT : public QUESTION_POSITION
{
    int value;
public:
    question_value play(LEVEL*, PLAYER* player);
    QUESTION_TELEPORT(int val, CHANNEL* channel);
};

// @@ class QUESTION_ZAP

class QUESTION_ZAP : public QUESTION_POSITION
{
    RAY* ray;
public:
    question_value play(LEVEL* level, PLAYER* player);
    QUESTION_ZAP(RAY* r, CHANNEL* channel);
    ~QUESTION_ZAP();
};

// @@ class QUESTION_BREATHE

class QUESTION_BREATHE : public QUESTION_POSITION
{
public:
    question_value play(LEVEL* level, PLAYER* player);
    QUESTION_BREATHE(CHANNEL* channel);
};

// @@ class QUESTION_STRING

class QUESTION_STRING : public QUESTION
{
protected:
    char string[MessageSize];
public:
    void parse(CHANNEL* channel);
    QUESTION_STRING(CHANNEL* channel, 
		    prompt_type prompt, request_type request = REQUEST_STRING);
};    


// @@ class QUESTION_RESTORE

class QUESTION_RESTORE : public QUESTION_STRING
{
    STRING name;
    QUESTION* TryAgain;
    SLEEP* sleep;
public:
    question_value play(LEVEL*, PLAYER* player);
    QUESTION* next();
    QUESTION_RESTORE(CHANNEL* channel, SLEEP* saved, const char* who);
};    


// @@ class QUESTION_START_UP

class QUESTION_START_UP : public QUESTION_STRING
{
    QUESTION* TryAgain;
    SLEEP* sleep;
public:
    question_value play(LEVEL* level, PLAYER* player);
    QUESTION* next()
    { return TryAgain; }
    QUESTION_START_UP(PLAYER* player, SLEEP* saved);
};    

// @@ class QUESTION_PASSWORD

class QUESTION_PASSWORD : public QUESTION_STRING
{
public:
    question_value play(LEVEL* level, PLAYER* player);
    QUESTION_PASSWORD(CHANNEL* channel);
};

// @@ end

#endif QUESTION_HEADER

// @@ emacs
//
// Local Variables:
// mode: c++
// outline-regexp: "// @+ "
// eval: (outline-minor-mode 1)
// End:
